"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (g && (g = 0, op[0] && (_ = 0)), _) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
var __spreadArray = (this && this.__spreadArray) || function (to, from, pack) {
    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {
        if (ar || !(i in from)) {
            if (!ar) ar = Array.prototype.slice.call(from, 0, i);
            ar[i] = from[i];
        }
    }
    return to.concat(ar || Array.prototype.slice.call(from));
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EventQueue = exports.EventTypes = void 0;
var Request_1 = require("./Request");
var utils_1 = require("./utils");
var chunk_1 = __importDefault(require("lodash/chunk"));
exports.EventTypes = {
    variableEvaluated: 'variableEvaluated',
    variableDefaulted: 'variableDefaulted',
};
var EventQueue = /** @class */ (function () {
    function EventQueue(sdkKey, dvcClient, options) {
        var _a, _b;
        this.eventQueueBatchSize = 100;
        this.sdkKey = sdkKey;
        this.client = dvcClient;
        this.eventQueue = [];
        this.aggregateEventMap = {};
        this.options = options;
        var eventFlushIntervalMS = typeof options.eventFlushIntervalMS === 'number'
            ? options.eventFlushIntervalMS
            : 10 * 1000;
        if (eventFlushIntervalMS < 500) {
            throw new Error("eventFlushIntervalMS: ".concat(eventFlushIntervalMS, " must be larger than 500ms"));
        }
        else if (eventFlushIntervalMS > 60 * 1000) {
            throw new Error("eventFlushIntervalMS: ".concat(eventFlushIntervalMS, " must be smaller than 1 minute"));
        }
        this.flushInterval = setInterval(this.flushEvents.bind(this), eventFlushIntervalMS);
        this.flushEventQueueSize = (_a = options === null || options === void 0 ? void 0 : options.flushEventQueueSize) !== null && _a !== void 0 ? _a : 100;
        this.maxEventQueueSize = (_b = options === null || options === void 0 ? void 0 : options.maxEventQueueSize) !== null && _b !== void 0 ? _b : 1000;
        if (this.flushEventQueueSize >= this.maxEventQueueSize) {
            throw new Error("flushEventQueueSize: ".concat(this.flushEventQueueSize, " must be smaller than ") +
                "maxEventQueueSize: ".concat(this.maxEventQueueSize));
        }
        else if (this.flushEventQueueSize < 10 ||
            this.flushEventQueueSize > 1000) {
            throw new Error("flushEventQueueSize: ".concat(this.flushEventQueueSize, " must be between 10 and 1000"));
        }
        else if (this.maxEventQueueSize < 100 ||
            this.maxEventQueueSize > 5000) {
            throw new Error("maxEventQueueSize: ".concat(this.maxEventQueueSize, " must be between 100 and 5000"));
        }
    }
    EventQueue.prototype.flushEvents = function () {
        return __awaiter(this, void 0, void 0, function () {
            var user, eventsToFlush, aggregateEventsToFlush, eventRequests, _i, eventRequests_1, eventRequest, res, ex_1;
            var _a;
            return __generator(this, function (_b) {
                switch (_b.label) {
                    case 0:
                        user = this.client.user;
                        if (!user) {
                            this.client.logger.warn('Skipping event flush, user has not been set yet.');
                            return [2 /*return*/];
                        }
                        eventsToFlush = __spreadArray([], this.eventQueue, true);
                        aggregateEventsToFlush = this.eventsFromAggregateEventMap();
                        eventsToFlush.push.apply(eventsToFlush, aggregateEventsToFlush);
                        if (!eventsToFlush.length) {
                            return [2 /*return*/];
                        }
                        this.client.logger.info("Flush ".concat(eventsToFlush.length, " Events"));
                        this.eventQueue = [];
                        this.aggregateEventMap = {};
                        eventRequests = (0, chunk_1.default)(eventsToFlush, this.eventQueueBatchSize);
                        _i = 0, eventRequests_1 = eventRequests;
                        _b.label = 1;
                    case 1:
                        if (!(_i < eventRequests_1.length)) return [3 /*break*/, 6];
                        eventRequest = eventRequests_1[_i];
                        _b.label = 2;
                    case 2:
                        _b.trys.push([2, 4, , 5]);
                        return [4 /*yield*/, (0, Request_1.publishEvents)(this.sdkKey, this.client.config || null, user, eventRequest, this.client.logger)];
                    case 3:
                        res = _b.sent();
                        if (res.status === 201) {
                            this.client.logger.info("DevCycle Flushed ".concat(eventRequest.length, " Events."));
                        }
                        else if (res.status >= 500 || res.status === 408) {
                            this.client.logger.warn('failed to flush events, retrying events. ' +
                                "Response status: ".concat(res.status, ", message: ").concat(res.statusText));
                            (_a = this.eventQueue).push.apply(_a, eventRequest);
                        }
                        else {
                            this.client.logger.error('failed to flush events, dropping events. ' +
                                "Response status: ".concat(res.status, ", message: ").concat(res.statusText));
                        }
                        return [3 /*break*/, 5];
                    case 4:
                        ex_1 = _b.sent();
                        this.client.eventEmitter.emitError(ex_1);
                        this.client.logger.error('failed to flush events due to error, dropping events. ' +
                            "Error message: ".concat(ex_1 === null || ex_1 === void 0 ? void 0 : ex_1.message));
                        return [3 /*break*/, 5];
                    case 5:
                        _i++;
                        return [3 /*break*/, 1];
                    case 6: return [2 /*return*/];
                }
            });
        });
    };
    /**
     * Queue DVCAPIEvent for producing
     */
    EventQueue.prototype.queueEvent = function (event) {
        if (this.checkEventQueueSize()) {
            this.client.logger.warn("DevCycle: Max event queue size (".concat(this.maxEventQueueSize, ") reached, dropping event: ").concat(event));
            return;
        }
        this.eventQueue.push(event);
    };
    /**
     * Queue DVCEvent that can be aggregated together, where multiple calls are aggregated
     * by incrementing the 'value' field.
     */
    EventQueue.prototype.queueAggregateEvent = function (event) {
        var _a;
        if (this.checkEventQueueSize()) {
            this.client.logger.warn("DevCycle: Max event queue size (".concat(this.maxEventQueueSize, ") reached, dropping event: ").concat(event));
            return;
        }
        (0, utils_1.checkParamDefined)('type', event.type);
        (0, utils_1.checkParamDefined)('target', event.target);
        event.date = Date.now();
        event.value = 1;
        var aggEventType = this.aggregateEventMap[event.type];
        if (!aggEventType) {
            this.aggregateEventMap[event.type] = (_a = {}, _a[event.target] = event, _a);
        }
        else if (aggEventType[event.target]) {
            aggEventType[event.target].value++;
        }
        else {
            aggEventType[event.target] = event;
        }
    };
    EventQueue.prototype.checkEventQueueSize = function () {
        var aggCount = Object.values(this.aggregateEventMap).reduce(function (acc, v) { return acc + Object.keys(v).length; }, 0);
        var queueSize = this.eventQueue.length + aggCount;
        if (queueSize >= this.flushEventQueueSize) {
            this.flushEvents();
        }
        return queueSize >= this.maxEventQueueSize;
    };
    /**
     * Turn the Aggregate Event Map into an Array of DVCAPIEvent objects for publishing.
     */
    EventQueue.prototype.eventsFromAggregateEventMap = function () {
        return Object.values(this.aggregateEventMap)
            .map(function (typeMap) { return Object.values(typeMap); })
            .flat();
    };
    EventQueue.prototype.close = function () {
        return __awaiter(this, void 0, void 0, function () {
            return __generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        clearInterval(this.flushInterval);
                        return [4 /*yield*/, this.flushEvents()];
                    case 1:
                        _a.sent();
                        return [2 /*return*/];
                }
            });
        });
    };
    return EventQueue;
}());
exports.EventQueue = EventQueue;
//# sourceMappingURL=EventQueue.js.map